#bib-dialog .ui-tabs-panel {
    min-height: 0;
    height: calc(100vh - 278px);
}

#bib-dialog .fw-dialog-table th {
    vertical-align: top;
    padding: 0 6px 0 8px;
    width: 164px;
}

#bib-dialog .entry-field .ProseMirror, #bib-dialog .entry-field input[type=text] {
    width: 686px;
}

#bib-dialog :focus {
  outline-color: transparent;
  outline: 0;
}

#bib-dialog .entry-field.fw-fomt-error input[type=text] {
    background-color: #f34141;
}


#bib-dialog .entry-field .selection-list {
    position: relative
}

#bib-dialog .entry-field select {
    padding: 0px 3px 0px 13px;
    border: 0 none;
    background-color: #f3f3f3;
    height: 38px;
    width: 561px;
    appearance: none;
    -webkit-appearance: none;
    -moz-appearance: none;
    cursor: pointer;
    display: inline-block;
}

#bib-dialog .entry-field .type-switch-input-inner select {
    width: 100%;
}

#bib-dialog .entry-field .fw-select-arrow {
    position: absolute;
	top: 14px;
	right: 16px;
	pointer-events: none;
}

#bib-dialog .type-switch {
    position: relative;
    width: 140px;
    height: 38px;
    float: left;
    margin-right: 5px;
    font-size: 14px;
    color: white;
    background-color: #76a698;
    border: 0 none;
    border-radius: 19px;
    white-space: nowrap;
    padding: 0;
    line-height: 38px;
    overflow: hidden;
    cursor: pointer;
}

#bib-dialog .type-switch.disabled {
    background-color: #e0e0e0;
}

.type-switch-inner {
    position: absolute;
    display: block;
    left: 0;
    top: 0;
    width: calc(200% - 38px);
    transition: left 0.3s;
}

.type-switch-inner:after {
    box-sizing: border-box;
    display: block;
    position: absolute;
    left: calc(50% - 19px);
    top: 0;
    content: "";
    width: 38px;
    height: 38px;
    border-radius: 19px;
    background-color: #f9f9f9;
    border: 1px solid #e0e0e0;
}

.value2 .type-switch-inner {
    left: calc(-100% + 38px)
}

.type-switch-label {
    box-sizing: border-box;
    display: block;
    width: 50%;
    float: left;
}

.type-switch-label:first-child {
    padding-right: 12px;
}

.type-switch-label:last-child {
    padding-left: 12px;
}

#bib-dialog .type-switch-input-wrapper {
    width: 686px;
}

.type-switch-input-wrapper:after {
    content: "";
    display: table;
    clear: both;
}

.type-switch-input-inner {
    position: relative;
    float: left;
    width: calc(100% - 145px);
}

#bib-dialog .type-switch-input-inner .field-part {
    display: inline-block;
    margin-right: 5px;
}

#bib-dialog .type-switch-input-inner .field-part:last-child {
    margin-right: 0;
}

#bib-dialog .type-switch-input-inner .field-part-huge .ProseMirror {
    width: 250px;
}

#bib-dialog .type-switch-input-inner .field-part-long .ProseMirror {
    width: 130px;
}

#bib-dialog .field-part-short .ProseMirror {
    width: 64px;
}

#bib-dialog .type-switch-input-inner .field-part-single {
    width: 100%;
}

#bib-dialog .type-switch-input-inner .field-part-single .ProseMirror {
    width: 100%;
}


#bib-dialog .ProseMirror-focused div.literal,
#bib-dialog .ProseMirror-focused pre.long-literal {
    background-color: #fbf9d2;
}

#bib-dialog .fw-pulldown.fw-center {
    left: 10px;
    top: 40px;
}

.tokenfield {
    width: 686px;
    position: relative;
}

.tokenfield:before, .tokenfield:after {
    content: " ";
    display: table;
}

.tokenfield:after {
    clear: both;
}

.tokenfield.tokenfield-mode-tokens {
    display: block;
    position: relative;
    font-size: 14px;
    line-height: 1.42857;
    color: #555555;
    background-color: #e0e0e0;
}

.tokenfield-set {
    position: relative;
    padding: 0 10px;
}

.tokenfield .tokenfield-set > ul {
    margin: 0;
    padding: 0;
    list-style-type: none;
}

.tokenfield .tokenfield-set > ul > li {
    display: inline-block;
    margin: 8px 5px 0 0;
    padding: 0 5px;
    border-radius: 3px;
    line-height: 1.5;
    cursor: pointer;
    color: #fff;
    background: #76a698;
    border: 1px solid #7d948e;
}

.tokenfield .tokenfield-set > ul > li:last-child {
    margin-bottom: 8px;
}

.tokenfield .tokenfield-set > ul > li a {
    color: #fff;
}

.tokenfield .tokenfield-input {
    outline: none;
}

div.literal, pre.long-literal {
    box-sizing: border-box;
    font-size: 14px;
    line-height: 1.33em;
    box-shadow: none;
    border-radius: 0;
    padding: 11px 10px 0;
    min-height: 38px;
    background-color: #f3f3f3;
}

pre.long-literal {
    min-height: 112px;
    height: auto;
}

div.literal strong, pre.long-literal strong  {
    font-weight: bold;
}

div.literal em, pre.long-literal em {
    font-style: italic;
}

.list-input-wrapper .add-remove-list {
    display: inline-block;
    cursor: pointer;
    margin-left: 14px;
}

.list-input-wrapper .add-remove-list:before {
    content: '\2796';
    width: 0.7em;
}

.list-input-wrapper:last-child .add-remove-list:before {
    content: '\2795';
}

#bib-dialog .date-form input {
    width: 66px;
    margin-right: 1px;
}

#bib-dialog .date-form .separator {
    display: inline-block;
    width: 15px;
    text-align: center;
}

#bib-dialog td .dk-selected, #bib-dialog td .dk-select-options {
    width: 486px !important;
}

#bib-dialog th .dk-selected, #bib-dialog th .dk-select-options {
    width: 140px !important;
}

#bib-dialog sup {
    vertical-align: super;
    font-size: smaller;
}

#bib-dialog sub {
    vertical-align: sub;
    font-size: smaller;
}

#bib-dialog .smallcaps {
    font-variant: small-caps;
}

#bib-dialog .nocase {
    text-decoration: underline;
}

.checkable-label {
    width: 462px;
    height: 36px;
    border: solid 1px #ccc;
    line-height: 36px;
    background-color: #fff;
    padding: 0 12px;
    font-size: 13px;
    margin-bottom: 10px;
}

#source-type-selection {
    position: relative;
    display: block;
    z-index: 2;
    margin-bottom: 18px;
    margin-right: 49px;
}

#source-type-selection .fw-pulldown > ul {
    max-height: 230px;
    overflow: scroll;
}

#selected-source-type-title {
    display: inline-block;
    white-space: nowrap;
    overflow: hidden;
}
#source-type-selection .icon-down-dir {
    position: absolute;
    right: 10px;
}

#editCategories {
    width: 369px !important;
}
#editCategories input[type="text"] {
    width: 310px;
}

/* api search */

#bibimport-api-search button.api-import {
    float: left;
    margin-right: 10px;
    margin-bottom: 10px;
}

#bibimport-api-search div.item {
    margin-bottom: 10px;
}


#bibimport-api-search h3 {
    font-weight: 700;
    margin: 10px 0px 10px 20px;
    font-size: 1.1em;
}

#bibimport-api-search .bibimport-search-result, #bibimport-api-search div.item {
    clear: left;
}

/**
Modal header
 */
.bib-dialog-header .fw-select-container {
    display: inline-block;
    vertical-align: middle;
    margin: 0;
}

.bib-dialog-header .ui-tabs-nav {
    display: inline-block;
    vertical-align: middle;
    margin: 0;
    height: auto;
}

.tab-link-inner {
    display: block;
    margin-left: 20px;
    color: rgba(98, 96, 96, 0.6);
}

.current-tab .tab-link-inner {
    color: #2c2723;
}