# Generated by Django 2.1.7 on 2019-03-16 02:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('style', '0001_squashed_0002_auto_20151226_1110'),
    ]

    operations = [
        migrations.AlterField(
            model_name='citationstyle',
            name='contents',
            field=models.TextField(default='', help_text='The XML style definiton.'),
        ),
        migrations.AlterField(
            model_name='citationstyle',
            name='short_title',
            field=models.SlugField(default='default', help_text='A title used for constant names.', max_length=40),
        ),
        migrations.AlterField(
            model_name='citationstyle',
            name='title',
            field=models.CharField(default='Default', help_text='The human readable title.', max_length=128),
        ),
        migrations.AlterField(
            model_name='documentstyle',
            name='contents',
            field=models.TextField(default='', help_text='The CSS style definiton.'),
        ),
        migrations.AlterField(
            model_name='documentstyle',
            name='filename',
            field=models.SlugField(default='default', help_text='The base of the filenames the style occupies.', max_length=20),
        ),
        migrations.AlterField(
            model_name='documentstyle',
            name='title',
            field=models.CharField(default='Default', help_text='The human readable title.', max_length=128),
        ),
    ]
