# Generated by Django 2.2.3 on 2019-08-03 05:49
from django.db import migrations, models


class Migration(migrations.Migration):
    
    dependencies = [
        ('style', '0002_auto_20190316_0241'),
    ]

    operations = [
        migrations.AlterField(
            model_name='citationstyle',
            name='contents',
            field=models.TextField(help_text='The style definiton (JSON, accepts also XML).'),
        ),
        migrations.AlterField(
            model_name='citationlocale',
            name='contents',
            field=models.TextField(help_text='The locale definiton (JSON, accepts also XML).'),
        )
    ]
