#template-editor table.title_id input[type='text'] {
    margin-bottom: 4px;
}

.bibliography-header-value input[type='text'] {
    margin: 10px;
    background-color: #fff;
    border: 1px solid rgb(169, 169, 169);
}

.bibliography-header-value table.fw-small {
    width: 100%;
}
