# Generated by Django 2.1.7 on 2019-02-27 21:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0006_auto_20190118_1342'),
    ]

    operations = [
        migrations.AlterField(
            model_name='accessright',
            name='rights',
            field=models.CharField(choices=[('write', 'Writer'), ('write-tracked', 'Write with tracked changes'), ('comment', 'Commentator'), ('review', 'Reviewer'), ('read', 'Reader'), ('read-without-comments', 'Reader without comment access')], max_length=21),
        ),
    ]
