.fw-basic-text {
    font-family: 'Lato', sans-serif;
    font-size: 13px;
    line-height: 1.33em;
    color: #2c2723;
    font-weight: 400;
}

a,
.fw-link-text {
    cursor: pointer;
    color: #b73717;
}

a:hover,
.fw-link-text:hover {
    cursor: pointer;
    color: #2c2723;
}

.fw-header-text {
    color: #ffffff;
    font-size: 14px;
}

.fw-header-navigation-text {
    color: #fff;
    font-weight: 400;
    letter-spacing: 0.6px;
    text-transform: uppercase;
    font-size: 15px;
}

.fw-big-title {
    font-size: 30px;
    font-weight: 300;
}

.fw-green-title {
    font-size: 14px;
    font-weight: 700;
    color: #84b4a7;
}

.fw-button-text {
    font-size: 14px;
    color: #000;
}

.fw-social-button-text {
    font-size: 15px;
    color: #fff;
}

.fw-login-button-text {
    font-size: 14px;
    color: #fff;
    text-transform: uppercase;
}

.fw-button-text-bold {
    font-size: 11px;
    font-weight: 900;
}

.fw-font-light {
    font-weight: 300;
}

.fw-font-bold {
    font-weight: 700;
}

.fw-font-ultra-bold {
    font-weight: 900;
}

.fw-form-text {
    font-family: 'Lato', sans-serif;
    font-size: 14px;
    color: #2c2723;
}

.fw-italic-title {
    font-style: italic;
    font-size: 22px;
    color: #fff;
}

.fw-text-white-shadow {
    text-shadow: 1px 1px 0 rgba(255,255,255,0.3);
}
