.fw-data-table {
    font-size: 13px;
    line-height: 1.25;
}

.fw-data-table thead,
.dataTable-table thead {
    font-weight: 900;
    font-size: 11px;
}

.dataTable-table thead th a {
    color: #2c2723;
}

.fw-data-table th,
.fw-data-table td,
.dataTable-table th,
.dataTable-table td {
    padding: 15px 0;
    vertical-align: top;
}

.fw-data-table .date,
.dataTable-table .date {
    white-space: nowrap;
}

.fw-data-table.fw-large tbody {
    width: 985px;
    max-height: calc(100vh - 320px);
}

.fw-data-table.fw-middle tbody {
    width: 600px;
    max-height: calc(100vh - 320px);
}

.fw-data-table.fw-small tbody {
    width: 342px;
    height: 270px;
}

.fw-data-table.fw-min tbody {
    width: 342px;
    height: 225px;
}

/*.fw-data-table tbody tr:hover {
    background-color: #fbf9d2;
}*/

.fw-data-table th,
.fw-data-table td,
.dataTable-table th,
.dataTable-table td {
    border-bottom: solid 1px #cecece;
}

.fw-data-table th:last-child,
.fw-data-table td:last-child,
.dataTable-table th:last-child,
.dataTable-table td:last-child {
    padding-right: 0;
}

.fw-data-table td.td-icon {
    text-align: center;
}

.fw-data-table-title {
    color: #a7321f;
}

.fw-data-table-title i.fa,
.fw-data-table-title i.far {
    margin-right: 10px;
}

.fw-data-table .icon-trash {
    font-size: 15px;
}

.owned-by-user .icon-access-right,.fw-data-table .fa-clock, .fw-data-table .fa-trash, .fw-data-table .fa-download {
    color: #a7321f;
    cursor: pointer;
}

.owned-by-user .icon-access-right:hover,.fw-data-table .fa-clock:hover, .fw-data-table .fa-trash:hover, .fw-data-table .fa-download:hover {
    color: #2c2723;
}

/* A deleted access rights row */
.collaborator-tr[data-right=delete] {
    display: none;
}

.fw-data-table .fw-avatar,
.fw-data-table .fw-string-avatar {
    display: inline-block;
    width: 25px;
    height: 25px;
    vertical-align: top;
    margin: -4px 10px -5px 0;
}

/*
 * Data Tables Plugin
 */
.fw-table-wrapper .dataTables_filter {
    position: fixed;
    left: auto;
    top: 96px;
    margin-left: 495px;
    width: 315px;
    text-align: right;
    z-index: 5;
}

.fw-table-wrapper .dataTables_filter label {
    color: #a9abaa;
}

.fw-table-wrapper .dataTables_filter label.focus {
    color: #000;
}

.fw-table-wrapper .dataTables_filter label:after {
    margin-left: 5px;
    font-size: 16px;
}

.fw-table-wrapper .dataTables_filter input[type="search"] {
    width: 255px;
    height: 20px;
    padding: 0 0 2px;
    border-style: none;
    border-bottom-style: solid;
    border-bottom-width: 1px;
}

.fw-table-wrapper .dataTables_filter input[type="search"]:focus {
    background-color: transparent;
}

/* the usermedia table */
.fw-usermedia-title {
    display: inline-block;
    height: auto;
}
.fw-usermedia-type {
    display: block;
    margin-top: 3px;
}
.fw-usermedia-image {
    display: inline-block;
    max-width: 60px;
    max-height: 60px;
    overflow: hidden;
    vertical-align: top;
    margin: 0 14px 2px 0;
}
.fw-usermedia-image > img {
    min-width: 100%;
    min-height: 100%;
}

/* Align text with top of images */
.fw-data-table tbody td {
    vertical-align: top;
}

.fw-data-table tbody td.checkable {
    position: relative;
}
.fw-data-table tbody tr.checked td.checkable:after {
    width: 1em;
    height: 1em;
    position: absolute;
    right: 12px;
    top: 12px;
}

/* Word counter dialog table */
.fw-word-counter-tbody {
    line-height: 38px;
}

/* Bulk action */
.dt-bulk {
    display: block;
    position: relative;
    min-width: 30px;
}

.dt-bulk .fw-pulldown {
    display: none;
}

.dt-bulk.opened .fw-pulldown {
    display: block;
}

.dt-bulk .fw-check + label {
    display: inline-block;
    vertical-align: middle;
}

.dt-bulk-dropdown {
    display: inline-block;
    vertical-align: middle;
    padding-left: 3px;
    cursor: pointer;
}
