# Generated by Django 2.2.4 on 2019-08-30 04:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0016_auto_20190829_2147'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='documenttemplate',
            name='citation_styles',
        ),
    ]
