# Generated by Django 2.2.4 on 2019-08-29 19:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0015_auto_20190828_1108'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='doc_version',
            field=models.DecimalField(decimal_places=1, default=3.1, max_digits=3),
        ),
        migrations.AlterField(
            model_name='documentrevision',
            name='doc_version',
            field=models.DecimalField(decimal_places=1, default=3.1, max_digits=3),
        ),
        migrations.AlterField(
            model_name='documenttemplate',
            name='doc_version',
            field=models.DecimalField(decimal_places=1, default=3.1, max_digits=3),
        ),
    ]
