# Generated by Django 2.2.3 on 2019-08-08 09:26

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0012_auto_20190704_1219'),
    ]

    operations = [
        migrations.AddField(
            model_name='documenttemplate',
            name='added',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='documenttemplate',
            name='updated',
            field=models.DateTimeField(auto_now=True),
        ),
    ]
