# Generated by Django 2.2.3 on 2019-07-04 12:19

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0011_auto_20190701_1744'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='template',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='document.DocumentTemplate'),
        ),
        migrations.CreateModel(
            name='AccessRightInvite',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('email', models.EmailField(max_length=254)),
                ('rights', models.CharField(choices=[('write', 'Writer'), ('write-tracked', 'Write with tracked changes'), ('comment', 'Commentator'), ('review', 'Reviewer'), ('read', 'Reader'), ('read-without-comments', 'Reader without comment access')], max_length=21)),
                ('document', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='document.Document')),
            ],
        ),
    ]
