# Generated by Django 2.2.2 on 2019-07-01 17:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0009_documentrevision_doc_version'),
        ('document', '0008_auto_20190701_0943'),
    ]

    operations = [
    ]
