.fw-button {
    display: inline-block;
    width: auto;
    height: 34px;
    color: #fff;
    line-height: 34px;
    padding: 0 12px;
    font-size: 14px;
    border-style: solid;
    border-width: 1px;
    border-radius: 3px;
    cursor: pointer;
    text-decoration: none;
    background: none;
    box-shadow: none;
    box-sizing: content-box;
}

.fw-button:hover {
    color: #fff;
}

.fw-button:focus {
    outline-width: 5px;
    outline-color: #8ab5a6;
}

.fw-button.fw-uppercase {
    text-transform: uppercase;
}

.fw-button.fw-large {
    height: 40px;
    line-height: 40px;
    padding: 0 10px;
}

.fw-button.fw-small {
    height: 26px;
    line-height: 26px;
    padding: 0 6px;
}

.fw-button.fw-square {
    width: 34px;
    padding: 0;
    text-align: center;
}
.fw-button.fw-large.fw-square {
    width: 40px;
}

.fw-button .fw-button-icon:before {
    line-height: 34px;
    margin: 0;
}
.fw-button.fw-large .fw-button-icon:before {
    line-height: 40px;
}
.fw-button.fw-small .fw-button-icon:before {
    line-height: 26px;
}

.fw-button.fw-light {
    border-color: #e0e0e0;
    background-color: #f9f9f9;
    color: #2c2723;
}

.fw-button.fw-light:not(.disabled):hover,
.fw-button.fw-light.ui-state-active {
    background-color: #e0e0e0;
    color: #2c2723;
}

.fw-button.fw-white {
    border: 0 none;
    background-color: #f3f3f3;
    color: #000;
}
.fw-button.fw-white:not(.disabled):hover {
    background-color: #f9f9f9;
    color: #000;
}

.fw-button.fw-green {
    border-color: #7d948e;
    background-color: #76a698;
}
.fw-button.fw-green:not(.disabled):hover {
    background-color: #96c6b8;
}

.fw-button.fw-orange {
    background: linear-gradient(to bottom left, #d3421c, #b13615);
    border-color: #7d331f;
}
.fw-button.fw-orange:not(.disabled):hover {
    background: #b13615;
}

.fw-button.fw-dark {
    background: linear-gradient(to bottom left, #6a6a6a, #585858);
    box-shadow: 0 1px 1px 0 #7f7f7f inset;
    border-color: #4f4d4f;
}
.fw-button.fw-dark:not(.disabled):hover {
    background: #585858;
}
.fw-button:disabled {
    background: grey;
    color: darkgrey;
    border-color: darkgrey;
}
.fw-button.fw-left {
    float: left;
}

.fw-button.fw-strong {
    font-weight: bold;
}

.fw-button.fw-em {
    font-style: italic;
}

.fw-button.fw-smallcaps {
    font-variant: small-caps;
}

.fw-button.fw-socialaccount {
    display: inline-block;
    width: auto;
    height: 38px;
    border-radius: 5px;
    font-size: 15px;
    vertical-align: top;
    line-height: 35px;
    padding: 0;
}

.fw-button.fw-socialaccount:hover {
    box-shadow: none;
}

.fw-button.fw-socialaccount > span {
    display: inline-block;
    width: 37px;
    height: 100%;
    text-align: center;
    font-size: 15px;
    margin-right: 12px;
    border-right-style: solid;
    border-right-width: 1px;
    line-height: 37px;
}

.fw-button.fw-socialaccount {
    background: linear-gradient(to bottom left, #0d78be, #0d63ab);
    box-shadow: 0px 0px 1px 1px #3c9fdf inset;
    border-color: #0557a1;
}

.fw-button.fw-socialaccount:hover {
    background: #0d63ab;
}

.fw-button.fw-socialaccount > span {
    border-right-color: #0557a1;
    box-shadow: 1px 0 0 0 #3c9fdf;
}

.fw-button.fw-facebook {
    background: linear-gradient(to bottom left, #0d78be, #0d63ab);
    box-shadow: 0px 0px 1px 1px #3c9fdf inset;
    border-color: #0557a1;
}
.fw-button.fw-facebook:hover {
    background: #0d63ab;
}
.fw-button.fw-facebook > span {
    border-right-color: #0557a1;
    box-shadow: 1px 0 0 0 #3c9fdf;
    font-size: 18px;
}

.fw-button.fw-twitter {
    background: linear-gradient(to bottom left, #3ccfff, #17addf);
    box-shadow: 0px 0px 1px 1px #3dcfff inset;
    border-color: #2498be;
}
.fw-button.fw-twitter:hover {
    background: #17addf;
}
.fw-button.fw-twitter > span {
    border-right-color: #2498be;
    box-shadow: 1px 0 0 0 #8ddffc;
}

.fw-button.fw-google {
    background: linear-gradient(to bottom left, #879ded, #6d84d8);
    box-shadow: 0px 0px 1px 1px #93a6e6 inset;
    border-color: #6578bb;
}
.fw-button.fw-google:hover {
    background: #6d84d8;
}
.fw-button.fw-google > span {
    border-right-color: #6578bb;
    box-shadow: 1px 0 0 0 #b1c0f2;
    font-size: 18px;
}

.fw-button.fw-google {
    background: linear-gradient(to bottom left, #879ded, #6d84d8);
    box-shadow: 0px 0px 1px 1px #93a6e6 inset;
    border-color: #6578bb;
}
.fw-button.fw-google:hover {
    background: #6d84d8;
}
.fw-button.fw-google > span {
    border-right-color: #6578bb;
    box-shadow: 1px 0 0 0 #b1c0f2;
    font-size: 18px;
}

.fw-button.fw-soundcloud {
    background: linear-gradient(to bottom left, #f76700, #fa3219);
    box-shadow: 0px 0px 1px 1px #f79955 inset;
    border-color: #c53421;
}
.fw-button.fw-soundcloud:hover {
    background: #fa1b19;
}
.fw-button.fw-soundcloud > span {
    border-right-color: #c53421;
    box-shadow: 1px 0 0 0 #f79955;
    font-size: 18px;
}

.fw-button.fw-openid {
    background: linear-gradient(to bottom left, #fae719, #fac219);
    box-shadow: 0px 0px 1px 1px #fad819 inset;
    border-color: #faad19;
}
.fw-button.fw-openid:hover {
    background: #e8c817;
}
.fw-button.fw-openid > span {
    border-right-color: #faad19;
    box-shadow: 1px 0 0 0 #fad819;
    font-size: 18px;
}

.fw-button.fw-persona {
    background: linear-gradient(to bottom left, #17e861, #1bbd54);
    box-shadow: 0px 0px 1px 1px #29e76c inset;
    border-color: #168e40;
}
.fw-button.fw-persona:hover {
    background: #17a74a;
}
.fw-button.fw-persona > span {
    border-right-color: #168e40;
    box-shadow: 1px 0 0 0 #29e76c;
    font-size: 18px;
}

.fw-button.fw-linkedin {
    background: linear-gradient(to bottom left, #c529e7, #ad22cb);
    box-shadow: 0px 0px 1px 1px #d648f4 inset;
    border-color: #841c9a;
}
.fw-button.fw-linkedin:hover {
    background: #9720b0;
}
.fw-button.fw-linkedin > span {
    border-right-color: #841c9a;
    box-shadow: 1px 0 0 0 #d648f4;
    font-size: 18px;
}

.fw-button.fw-github {
    background: linear-gradient(to bottom left, #f95858, #f92958);
    box-shadow: 0px 0px 1px 1px #f87795 inset;
    border-color: #d20e3c;
}
.fw-button.fw-github:hover {
    background: #e32653;
}
.fw-button.fw-github > span {
    border-right-color: #d20e3c;
    box-shadow: 1px 0 0 0 #f87795;
    font-size: 18px;
}

.fw-button.fw-stackexchange {
    background: linear-gradient(to bottom left, #fcff23, #eaed0d);
    box-shadow: 0px 0px 1px 1px #fdff91 inset;
    border-color: #ced03f;
    color: #585854;
}
.fw-button.fw-stackexchange:hover {
    background: #fcff00;
}
.fw-button.fw-stackexchange > span {
    border-right-color: #ced03f;
    box-shadow: 1px 0 0 0 #fdff91;
    font-size: 18px;
}

.fw-plain-button {
    display: block;
    font-size: 11px;
    line-height: 24px;
    background-color: #d3d3d3;
    font-weight: 700;
    text-align: center;
}

.fw-plain-button:hover {
    background-color: #e0e0e0;
}

.fw-button.disabled {
    color: #cccccc;
    cursor: default;
}

.fw-add-button:after {
    margin-left: 10px;
}

#comment-filter-dropdown {
    width: 100px;
}
